from typing import Callable, Union

from .http_error import HttpError
from .http_request import HttpRequest
from .http_response import HttpResponse
from .middleware import Middleware, MiddlewareHandler
from .routing import Route, Router
from .serverless.serverless import ServerlessFunction


class RouterMiddleware(Middleware):
    def __init__(self, router: Router):
        self.router = router

    def handle(self, request: HttpRequest, next: MiddlewareHandler) -> HttpResponse:
        try:
            route, controller = self.router.match(
                request.path, request.method
            )  # type: Route, Union[Callable, ServerlessFunction]

            request.path_parameters = route.parameters
            request.route = route

            response: HttpResponse
            if isinstance(controller, ServerlessFunction):
                response = controller.function(request)
            else:
                response = controller(request)

            return response
        except HttpError as error:
            return HttpResponse(status=error.status_code, body=error.http_message)


__all__ = ["RouterMiddleware"]



from .application import Application
from .http_cookies import (
    HttpCookie,
    HttpCookieError,
    HttpCookieJar,
    HttpCookieSameSitePolicy,
)
from .http_error import HttpError, NotFoundError
from .http_headers import HttpHeaders
from .http_message import (
    CompositeHttpMessage,
    FormHttpMessage,
    HttpMessage,
    JsonHttpMessage,
    MultipartHttpMessage,
    YamlHttpMessage,
)
from .http_method import HttpMethod
from .http_multipart_message_parser import UploadedFile, parse_multipart_message
from .http_query_string import HttpQueryString
from .http_request import HttpRequest
from .http_response import HttpResponse
from .http_status import HttpStatus
from .router_middleware import RouterMiddleware
from .routing import Route, Router
from .wsgi import create_wsgi_handler, serve
